<?php


require_once join(DIRECTORY_SEPARATOR, ["..", "..", "..", "init.php"]);
require_once join(DIRECTORY_SEPARATOR, [ROOTDIR, "vendor", "aspnix.com", "Helpers.php"]);
require_once join(DIRECTORY_SEPARATOR, [ROOTDIR, "modules", "gateways", "coinbase", "functions.php"]);
App::load_function("functions");
App::load_function("gateway");
App::load_function("invoice");
define("GATEWAY_MODULE", "coinbase");
$gateway = ASPnix_Helpers::getGatewaySettings(GATEWAY_MODULE);
$api_key = $gateway["api_key"];
$api_secret = $gateway["api_secret"];
$gateway_name = $gateway["name"];
$raw_response = file_get_contents("php://input");
$response = json_decode($raw_response, true);
$signature = isset($_SERVER["HTTP_X_CC_WEBHOOK_SIGNATURE"]) ? $_SERVER["HTTP_X_CC_WEBHOOK_SIGNATURE"] : NULL;
if (empty($signature)) {
    logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Missing HTTP Signature");
    http_response_code(406);
    exit;
}
$client = new ASPnix_CoinbaseClient($api_key, $api_secret);
if (!$client->verifyCallbackSignature($raw_response, $signature)) {
    logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Signature Mismatch");
    http_response_code(406);
    exit;
}
$event_id = $response["event"]["id"];
$order_id = $response["event"]["data"]["code"];
$type = $response["event"]["type"];
$charge_type = $response["event"]["data"]["metadata"]["charge_type"];
$invoice_id = $response["event"]["data"]["metadata"]["invoice_id"];
$user_id = $response["event"]["data"]["metadata"]["user_id"];
if ($event_id === ASPnix_CoinbaseClient::COINBASE_WEBHOOK_TEST_EVENTID) {
    if (in_array($type, [ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_PENDING, ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_COMFIRMED, ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_FAILED, ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_DELAYED])) {
        logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Webhook Test Success");
        http_response_code(200);
        exit;
    }
    logTransaction($gateway_name, "Unsupported webhook type (" . $type . "), please disable this webhook under your Coinbase Commerce account.", "Webhook Test Failed");
    http_response_code(406);
    exit;
}
checkCbInvoiceID($invoice_id, $gateway_name);
if (!array_key_exists("payments", $response["event"]["data"]) || empty($response["event"]["data"]["payments"])) {
    logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "No Payments Found");
    http_response_code(200);
    exit;
}
$payment = end($response["event"]["data"]["payments"]);
$amount = $payment["value"]["local"]["amount"];
$status = $payment["status"];
$transaction_id = $payment["transaction_id"];
$crypto_currency = $payment["value"]["crypto"]["currency"];
$local_currency = $payment["value"]["local"]["currency"];
checkCbTransID($transaction_id);
$invoice = ASPnix_Helpers::getInvoice($invoice_id);
$currency = ASPnix_Helpers::getCurrencyByCode($local_currency);
if ($type === ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_PENDING) {
    if ($invoice["balance"] <= $amount && ($invoice["status"] === ASPnix_Helpers::INVOICE_STATUS_UNPAID || $invoice["status"] === ASPnix_Helpers::INVOICE_STATUS_COLLECTIONS)) {
        ASPnix_Helpers::setInvoiceStatus($invoice_id, ASPnix_Helpers::INVOICE_STATUS_PENDING);
        ASPnix_Helpers::addTransactionHistory($invoice_id, $gateway_name, $transaction_id, "Payment Pending", 0, "", $amount, $currency["id"]);
        logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Payment Pending");
        http_response_code(200);
        exit;
    }
} else {
    if ($type === ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_COMFIRMED) {
        addInvoicePayment($invoice_id, $transaction_id, $amount, 0, GATEWAY_MODULE);
        logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Successful");
        ASPnix_CoinbaseHelper::saveTransaction($transaction_id, $order_id, $invoice_id);
        http_response_code(200);
        exit;
    }
    if ($type === ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_DELAYED) {
        ASPnix_Helpers::addTransactionHistory($invoice_id, $gateway_name, $transaction_id, "Payment Delayed", 0, "", $amount, $currency["id"]);
        logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Payment Delayed");
        http_response_code(200);
        exit;
    }
    if ($type === ASPnix_CoinbaseClient::COINBASE_WEBHOOK_CHARGETYPE_FAILED) {
        if ($invoice["balance"] <= $amount && $invoice["status"] === ASPnix_Helpers::INVOICE_STATUS_PENDING) {
            ASPnix_Helpers::setInvoiceStatus($invoice_id, ASPnix_Helpers::INVOICE_STATUS_UNPAID);
            ASPnix_Helpers::addTransactionHistory($invoice_id, $gateway_name, $transaction_id, "Payment Failed", 0, "", $amount, $currency["id"]);
            logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Payment Failed");
            http_response_code(200);
            exit;
        }
    } else {
        logTransaction($gateway_name, ASPnix_CoinbaseHelper::convertRawResponse($raw_response), "Unknown Webhook");
        http_response_code(406);
    }
}

?>